﻿// This is a "safe" class, meaning that it is created once 
// and never overwritten. Any custom code you add to this class 
// will be preserved when you regenerate your application.
//
// Typical customizations that may be done in this class include
//  - adding custom event handlers
//  - overriding base class methods

using System;
using System.Runtime;
using System.Data.SqlTypes;
using BaseClasses;
using BaseClasses.Data;
using BaseClasses.Data.SqlProvider;

namespace Warehouse.Business
{

/// <summary>
/// Provides access to the data in a database record from a table (or view).
/// Also provides access to the <see cref="Import_itemTable"></see> that each record is associated with.
/// </summary>
/// <remarks>
/// <para>
/// This is a "safe" class, meaning that it is generated once and never overwritten. 
/// Any changes you make to this class will be preserved when you regenerate your application.
/// </para>
/// </remarks>
/// <seealso cref="Import_itemTable"></seealso>
[SerializableAttribute()]
public class Import_itemRecord : BaseImport_itemRecord
{
		public Import_itemRecord()
		{
		#region "Code Customization"
		
		    // The following line will be inserted inside the
		    // constructor for table record class.
		    this.UpdatingRecord += 
		        new BaseClasses.IRecordWithTriggerEvents.UpdatingRecordEventHandler(this.Audit_Update);
		    
		#endregion
		
		#region "Code Customization"
		
		    // The following line will be inserted inside the
		    // constructor for table record class.
		    this.InsertingRecord += 
		        new BaseClasses.IRecordWithTriggerEvents.InsertingRecordEventHandler(this.Audit_Insert); 
		    
		#endregion
		}

		#region "Code Customization"
		
		/// <summary>
		/// This custom method is an event handler for the update event in the Data Access layer.
		/// </summary>
		private void Audit_Update(object sender, System.ComponentModel.CancelEventArgs e)
		{
            DataAccessSettings dataAccessSettingsObj = DataAccessSettings.Current;
		    string s;
		    s = (this.Napló + "\nMódosítva: " + DateTime.Now.ToString()).Trim()+": "+(string)dataAccessSettingsObj["UserID"];
		    this.Napló = s;
		}
		    
		#endregion
		
		#region "Code Customization"
		
		/// <summary>
		/// This custom method is an event handler for the insert event in the Data Access layer.
		/// </summary>
		private void Audit_Insert(object sender, System.ComponentModel.CancelEventArgs e)
		{
            DataAccessSettings dataAccessSettingsObj = DataAccessSettings.Current;
		    string s;
		    s = (this.Napló + "\nHozzáadva: " + DateTime.Now.ToString()).Trim()+": "+(string)dataAccessSettingsObj["UserID"];
		    this.Napló = s;
		}
		    
		#endregion
		} // End class Import_itemRecord

}
