﻿// This is a "safe" class, meaning that it is created once 
// and never overwritten. Any custom code you add to this class 
// will be preserved when you regenerate your application.
//
// Typical customizations that may be done in this class include
//  - adding custom event handlers
//  - overriding base class methods

using System;
using System.Data.SqlTypes;
using BaseClasses;
using BaseClasses.Data;
using BaseClasses.Data.SqlProvider;

namespace Warehouse.Business
{

/// <summary>
/// Provides access to the schema information and record data of a database table (or view).
/// See <see cref="BaseImport_logTable"></see> for additional information.
/// </summary>
/// <remarks>
/// See <see cref="BaseImport_logTable"></see> for additional information.
/// <para>
/// This class is implemented using the Singleton design pattern.
/// </para>
/// </remarks>
/// <seealso cref="BaseImport_logTable"></seealso>
/// <seealso cref="BaseImport_logSqlTable"></seealso>
/// <seealso cref="Import_logSqlTable"></seealso>
/// <seealso cref="Import_logDefinition"></seealso>
/// <seealso cref="Import_logRecord"></seealso>
/// <seealso cref="BaseImport_logRecord"></seealso>
[SerializableAttribute()]
public class Import_logTable : BaseImport_logTable, System.Runtime.Serialization.ISerializable, ISingleton
{

#region "ISerializable Members"

    /// <summary>
    /// Overridden to use the <see cref="Import_logTable_SerializationHelper"></see> class 
    /// for deserialization of <see cref="Import_logTable"></see> data.
    /// </summary>
    /// <remarks>
    /// Since the <see cref="Import_logTable"></see> class is implemented using the Singleton design pattern, 
    /// this method must be overridden to prevent additional instances from being created during deserialization.
    /// </remarks>
    void System.Runtime.Serialization.ISerializable.GetObjectData(
        System.Runtime.Serialization.SerializationInfo info, 
        System.Runtime.Serialization.StreamingContext context)
    {
        info.SetType(typeof(Import_logTable_SerializationHelper)); //No other values need to be added
    }

#region "Class Import_logTable_SerializationHelper"

    [SerializableAttribute()]
    private class Import_logTable_SerializationHelper: System.Runtime.Serialization.IObjectReference
    {
        //Method called after this object is deserialized
        public virtual object GetRealObject(System.Runtime.Serialization.StreamingContext context)
        {
            return Import_logTable.Instance;
        }
    }

#endregion

#endregion

    /// <summary>
    /// References the only instance of the <see cref="Import_logTable"></see> class.
    /// </summary>
    /// <remarks>
    /// Since the <see cref="Import_logTable"></see> class is implemented using the Singleton design pattern, 
    /// this field is the only way to access an instance of the class.
    /// </remarks>
    public readonly static Import_logTable Instance = new Import_logTable();

    private Import_logTable()
    {
    }


} // End class Import_logTable

}
