﻿
// This file implements the code-behind class for SignOut.aspx.
// App_Code\SignOut.Controls.vb contains the Table, Row and Record control classes
// for the page.  Best practices calls for overriding methods in the Row or Record control classes.

#region "Using statements"    

using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
        
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using BaseClasses;
using BaseClasses.Utils;
using BaseClasses.Data;
using BaseClasses.Data.SqlProvider;
using BaseClasses.Web.UI.WebControls;
        
using Warehouse.Business;
using Warehouse.Data;
        

#endregion

  
namespace Warehouse.UI
{
  
partial class SignOut
        : BaseApplicationPage
// Code-behind class for the SignOut page.
// Place your customizations in Section 1. Do not modify Section 2.
{
        


      #region "Section 1: Place your customizations here."

      public SignOut()
        {
            this.Initialize();
        }

        public void LoadData()
        {
            // LoadData reads database data and assigns it to UI controls.
            // Customize by adding code before or after the call to LoadData_Base()
            // or replace the call to LoadData_Base().
            LoadData_Base();
        }
      
      public override void SaveData()
      {
          this.SaveData_Base();
      }          
    
#region "Ajax Functions"

        [System.Web.Services.WebMethod()]
        public static Object[] GetRecordFieldValue(string tableName , 
                                                    string recordID , 
                                                    string columnName, 
                                                    string title, 
                                                    bool persist, 
                                                    int popupWindowHeight, 
                                                    int popupWindowWidth, 
                                                    bool popupWindowScrollBar)
        {
            // GetRecordFieldValue gets the pop up window content from the column specified by
            // columnName in the record specified by the recordID in data base table specified by tableName.
            // Customize by adding code before or after the call to  GetRecordFieldValue_Base()
            // or replace the call to  GetRecordFieldValue_Base().

            return GetRecordFieldValue_Base(tableName, recordID, columnName, title, persist, popupWindowHeight, popupWindowWidth, popupWindowScrollBar);
        }

        [System.Web.Services.WebMethod()]
        public static object[] GetImage(string tableName,
                                        string recordID, 
                                        string columnName, 
                                        string title, 
                                        bool persist, 
                                        int popupWindowHeight, 
                                        int popupWindowWidth, 
                                        bool popupWindowScrollBar)
        {
            // GetImage gets the Image url for the image in the column "columnName" and
            // in the record specified by recordID in data base table specified by tableName.
            // Customize by adding code before or after the call to  GetImage_Base()
            // or replace the call to  GetImage_Base().
            return GetImage_Base(tableName, recordID, columnName, title, persist, popupWindowHeight, popupWindowWidth, popupWindowScrollBar);
        }
        
    
      protected override void BasePage_PreRender(object sender, EventArgs e)
      {
          base.BasePage_PreRender(sender, e);
          Base_RegisterPostback();
      }
    
      
      #endregion

      // Page Event Handlers - buttons, sort, links
      
        public void ForgetSignInButton_Click(object sender, EventArgs args)
        {
            
          // Click handler for ForgetSignInButton.
          // Customize by adding code before the call or replace the call to the Base function with your own code.
          ForgetSignInButton_Click_Base(sender, args);
          // NOTE: If the Base function redirects to another page, any code here will not be executed.
        }
        
        public void OKButton_Click(object sender, EventArgs args)
        {
            
          // Click handler for OKButton.
          // Customize by adding code before the call or replace the call to the Base function with your own code.
          OKButton_Click_Base(sender, args);
          // NOTE: If the Base function redirects to another page, any code here will not be executed.
        }
        


#endregion

#region "Section 2: Do not modify this section."

      
        private void Initialize()
        {
            // Called by the class constructor to initialize event handlers for Init and Load
            // You can customize by modifying the constructor in Section 1.
            this.Init += new EventHandler(Page_InitializeEventHandlers);
            this.Load += new EventHandler(Page_Load);

        }
        
    
         private static string cultureui = "";
    
      // Handles base.Init. Registers event handler for any button, sort or links.
      // You can add additional Init handlers in Section 1.
      protected virtual void Page_InitializeEventHandlers(object sender, System.EventArgs e)
      {
	  		

            // Register the Event handler for any Events.
      
              this.ForgetSignInButton.Button.Click += new EventHandler(ForgetSignInButton_Click);
        
              this.OKButton.Button.Click += new EventHandler(OKButton_Click);
        
        }

        private void Base_RegisterPostback()
        {
                
        }

        // Handles base.Load.  Read database data and put into the UI controls.
        // You can add additional Load handlers in Section 1.
        protected virtual void Page_Load(object sender, EventArgs e)
        {
          
            //Event handler for cookie
            base.PreRender += new EventHandler(SignOut_PreRender);
    
             if (!this.IsPostBack)
             {
            
                    // Setup the header text for the validation summary control.
                    this.ValidationSummary1.HeaderText = GetResourceValue("ValidationSummaryHeaderText", "Warehouse");
                 
                     this.StateInit();
                     cultureui = (string)(Session["AppCultureUI"]);
                     ((BaseApplicationPage)(this.Page)).CurrentSecurity.Logout((BaseApplicationPage)(this.Page));
             }
                // Load data only when displaying the page for the first time or if postback from child window
            if ((!this.IsPostBack) || ( this.Request["__EVENTTARGET"] == "ChildWindowPostBack")) {   

                // Read the data for all controls on the page.
                // To change the behavior, override the DataBind method for the individual
                // record or table UI controls.
                this.LoadData();
                }
                
            
            }

            public static object[] GetRecordFieldValue_Base(string tableName ,
            string recordID ,
            string columnName,
            string title,
            bool persist,
            int popupWindowHeight,
            int popupWindowWidth,
            bool popupWindowScrollBar)
            {
            if (recordID != ""){
                recordID = System.Web.HttpUtility.UrlDecode(recordID);
                }
            string content = BaseClasses.Utils.MiscUtils.GetFieldData(tableName, recordID, columnName);
            
            content =  NetUtils.EncodeStringForHtmlDisplay(content);
            // returnValue is an array of string values.
            // returnValue(0) represents title of the pop up window.
            // returnValue(1) represents content ie, image url.
            // returnValue(2) represents whether pop up window should be made persistant
            // or it should close as soon as mouse moves out.
            // returnValue(3), (4) represents pop up window height and width respectivly
            // returnValue(5) represents whether pop up window should contain scroll bar.
            // (0),(2),(3) and (4) is initially set as pass through attribute.
            // They can be modified by going to Attribute tab of the properties window of the control in aspx page.
            object[] returnValue = new object[6];
            returnValue[0] = title;
            returnValue[1] = content;
            returnValue[2] = persist;
            returnValue[3] = popupWindowWidth;
            returnValue[4] = popupWindowHeight;
            returnValue[5] = popupWindowScrollBar;
            return returnValue;
        }

        public static object[] GetImage_Base(string tableName,
        string recordID,
        string columnName,
        string title,
        bool persist,
        int popupWindowHeight,
        int popupWindowWidth,
        bool popupWindowScrollBar)
        {
            string  content= "<IMG src =" + "\"../Shared/ExportFieldValue.aspx?Table=" + tableName + "&Field=" + columnName + "&Record=" + recordID + "\"/>";
        // returnValue is an array of string values.
        // returnValue(0) represents title of the pop up window.
        // returnValue(1) represents content ie, image url.
        // returnValue(2) represents whether pop up window should be made persistant
        // or it should close as soon as mouse moves out.
        // returnValue(3), (4) represents pop up window height and width respectivly
        // returnValue(5) represents whether pop up window should contain scroll bar.
        // (0),(2),(3), (4) and (5) is initially set as pass through attribute.
        // They can be modified by going to Attribute tab of the properties window of the control in aspx page.
        object[] returnValue = new object[6];
        returnValue[0] = title;
        returnValue[1] = content;
        returnValue[2] = persist;
        returnValue[3] = popupWindowWidth;
        returnValue[4] = popupWindowHeight;
        returnValue[5] = popupWindowScrollBar;
        return returnValue;
    }

    
      
      public void SaveData_Base()
      {
      
      }
      
      
    //sec_comment: this method control switches off AutoLogin at sign out
    public void SignOut_PreRender(object sender, System.EventArgs e) {
    
        UI.SignInState state = new SignInState();
        this.ForgetSignInButton.Button.Enabled = (state.IsUNToRemember || state.IsPassToRemember);
        string isAutoLogin = BaseClasses.Utils.NetUtils.GetCookie(NetUtils.CookieAutoLogin());
        if((isAutoLogin==null) || (isAutoLogin == "")) {
            return; }
        else if(isAutoLogin.ToLower() == "true") {
            BaseClasses.Utils.NetUtils.SetCookie(NetUtils.CookieAutoLogin(), "False");
        }                
    }
    
    private void StateInit() {
        UI.SignInState state = new UI.SignInState();
        string rUser = BaseClasses.Utils.NetUtils.GetCookie(NetUtils.CookieRememberName());
        if ((rUser != null) && (rUser.ToLower() == "true")) {
            state.IsUNToRemember = true;
        }
        else {
            state.IsUNToRemember = false;
        }
        string rPassword = BaseClasses.Utils.NetUtils.GetCookie(NetUtils.CookieRememberPassword());
        if ((rPassword != null) && (rPassword.ToLower() == "true")) {
            state.IsPassToRemember = true;
        }
        else {
            state.IsPassToRemember = false;
        }
    }
  

    // Load data from database into UI controls.
    // Modify LoadData in Section 1 above to customize.  Or override DataBind() in
    // the individual table and record controls to customize.
    public void LoadData_Base()
    {
    
            try {
                
                  // Must start a transaction before performing database operations
                  DbUtils.StartTransaction();

                  // Load data for each record and table UI control.
                  // Ordering is important because child controls get 
                  // their parent ids from their parent UI controls.
                
                        
                  this.DataBind();
                    
            
            } catch (Exception ex) {
                // An error has occured so display an error message.
                BaseClasses.Utils.MiscUtils.RegisterJScriptAlert(this, "Page_Load_Error_Message", ex.Message);
            } finally {
                if (!this.IsPostBack) {
                    // End database transaction
                    DbUtils.EndTransaction();
                }
            }
      
        }

        // Write out event methods for the page events
        
        // event handler for Button with Layout
        public void ForgetSignInButton_Click_Base(object sender, EventArgs args)
        {
              
              SignInState state = new SignInState();
              BaseClasses.Utils.NetUtils.SetCookie(NetUtils.CookieUserName(), "");
              BaseClasses.Utils.NetUtils.SetCookie(NetUtils.CookiePassword(), "");
              state.IsUNToRemember = false;
              state.IsPassToRemember = false;
              BaseClasses.Utils.NetUtils.SetCookie(NetUtils.CookieRememberName(), Boolean.FalseString);
              BaseClasses.Utils.NetUtils.SetCookie(NetUtils.CookieRememberPassword(), Boolean.FalseString);
            try {
                
            } catch (Exception ex) {
                  this.ErrorOnPage = true;
      
			// Report the error message to the end user
            BaseClasses.Utils.MiscUtils.RegisterJScriptAlert(this, "BUTTON_CLICK_MESSAGE", ex.Message);
          
            } finally {
    
            }
    
        }
            
        // event handler for Button with Layout
        public void OKButton_Click_Base(object sender, EventArgs args)
        {
              
            string url = @"../Default.aspx";
		    // The redirect URL is set on the Properties, Bindings.
			// The ModifyRedirectURL call resolves the parameters before the
			// Response.Redirect redirects the page to the URL.  
			// Any code after the Response.Redirect call will not be executed, since the page is
			// redirected to the URL.
            bool shouldRedirect = true;
            string TargetKey;
            string tk = null;
            TargetKey = tk;
            
            try {
                // Enclose all database retrieval/update code within a Transaction boundary
				DbUtils.StartTransaction();
				
                url = this.ModifyRedirectUrl(url, "",false);
                        this.CommitTransaction(sender);
    
      
            } catch (Exception ex) {
                  // Upon error, rollback the transaction
                  this.RollBackTransaction(sender);
                  shouldRedirect = false;
                  this.ErrorOnPage = true;
      
			// Report the error message to the end user
            BaseClasses.Utils.MiscUtils.RegisterJScriptAlert(this, "BUTTON_CLICK_MESSAGE", ex.Message);
          
            } finally {
                DbUtils.EndTransaction();
            }
            if (shouldRedirect) {
                this.Response.Redirect(url);
            }
        
        }
            
      


#endregion

  
}
  
}
  